/**
  ******************************************************************************
  * @file          : app_x-cube-tof1.c
  * @author        : IMG SW Application Team
  * @brief         : This file provides code for the configuration
  *                      of the STMicroelectronics.X-CUBE-TOF1.2.0.0 instances.
  ******************************************************************************
  *
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "app_x-cube-tof1.h"
#include "main.h"
#include <stdio.h>

#include "53l5a1_ranging_sensor.h"
#include "app_tof_pin_conf.h"
#include "stm32f4xx_nucleo.h"

#include "NanoEdgeAI.h"
#include "knowledge.h"

/*
 * The purpose of this sample application is to showcase the threshold detection
 * functionality of this device.
 *
 * When the device detects a target that match the configuration an IT is generated
 * and the host will start printing the measurement information on the serial connection (UART).
 *
 * Here is the default configuration:
 *
 * ITConfig.Criteria = RS_IT_IN_WINDOW;
 * ITConfig.LowThreshold = 200; // distance in mm
 * ITConfig.HighThreshold = 600; // distance in mm
 *
 * Other availables interrupt generation criteria for this device are:
 *
 * - RS_IT_DEFAULT          // IT if new measurement is ready (no thresholds)
 * - RS_IT_IN_WINDOW        // IT if distance > thresh_high
 * - RS_IT_OUT_OF_WINDOW    // IT if distance < LowThreshold OR distance > HighThreshold
 * - RS_IT_BELOW_LOW        // IT if distance <= LowThreshold
 * - RS_IT_ABOVE_HIGH       // IT if distance > HighThreshold
 * - RS_IT_EQUAL_LOW        // IT if distance == LowThreshold
 * - RS_IT_NOT_EQUAL_LOW    // IT if distance != LowThreshold
 */

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
#define NEAI_MODE					1 	/* 0: data logging mode, 1: NEAI functions mode */
#define SUCCESSIVE_MATRIX	8 	/* How many successive matrix we get in our neai_buffer */
#define MATRIX_RESOLUTION	64 	/* 64: when resolution 8x8, 16: when 4x4 */

/* Private variables ---------------------------------------------------------*/
static RANGING_SENSOR_Capabilities_t Cap;
static RANGING_SENSOR_ProfileConfig_t Profile;
static int32_t status = 0;
volatile uint8_t ToF_EventDetected = 0;
/* NEAI part */
uint16_t id_class = 0; // Point to id class (see argument of neai_classification fct)
float neai_buffer[SUCCESSIVE_MATRIX * MATRIX_RESOLUTION] = {0};
float output_class_buffer[CLASS_NUMBER]; // Buffer of class probabilities
const char *id2class[CLASS_NUMBER + 1] = { // Buffer for mapping class id to class name
	"unknown",
	"static",
	"to_the_left",
	"to_the_right",
};

/* Private function prototypes -----------------------------------------------*/
static void MX_53L5A1_ThresholdDetection_Init(void);
static void MX_53L5A1_ThresholdDetection_Process(void);
static void print_result(RANGING_SENSOR_Result_t *Result);

void MX_X_CUBE_TOF1_Init(void)
{
  /* USER CODE BEGIN SV */

  /* USER CODE END SV */

  /* USER CODE BEGIN X_CUBE_TOF1_Init_PreTreatment */

  /* USER CODE END X_CUBE_TOF1_Init_PreTreatment */

  /* Initialize the peripherals and the TOF components */

  MX_53L5A1_ThresholdDetection_Init();

  /* USER CODE BEGIN X_CUBE_TOF1_Init_PostTreatment */

  /* USER CODE END X_CUBE_TOF1_Init_PostTreatment */
}

/*
 * LM background task
 */
void MX_X_CUBE_TOF1_Process(void)
{
  /* USER CODE BEGIN X_CUBE_TOF1_Process_PreTreatment */

  /* USER CODE END X_CUBE_TOF1_Process_PreTreatment */

  MX_53L5A1_ThresholdDetection_Process();

  /* USER CODE BEGIN X_CUBE_TOF1_Process_PostTreatment */

  /* USER CODE END X_CUBE_TOF1_Process_PostTreatment */
}

static void MX_53L5A1_ThresholdDetection_Init(void)
{
  /* Initialize Virtual COM Port */
  BSP_COM_Init(COM1);

  status = VL53L5A1_RANGING_SENSOR_Init(VL53L5A1_DEV_CENTER);

  if (status != BSP_ERROR_NONE)
  {
    printf("VL53L5A1_RANGING_SENSOR_Init failed\n");
    while(1);
  }
}

static void MX_53L5A1_ThresholdDetection_Process(void)
{
  uint32_t Id;
  uint8_t zones_per_line;

  RANGING_SENSOR_ITConfig_t ITConfig;
  RANGING_SENSOR_Result_t Result;

  VL53L5A1_RANGING_SENSOR_ReadID(VL53L5A1_DEV_CENTER, &Id);
  VL53L5A1_RANGING_SENSOR_GetCapabilities(VL53L5A1_DEV_CENTER, &Cap);

  Profile.RangingProfile = RS_PROFILE_8x8_CONTINUOUS;
  Profile.TimingBudget = 5; /* 5 ms < TimingBudget < 100 ms */
  Profile.Frequency = 15; /* Hz */
  Profile.EnableAmbient = 0; /* Enable: 1, Disable: 0 */
  Profile.EnableSignal = 0; /* Enable: 1, Disable: 0 */

  /* set the profile if different from default one */
  VL53L5A1_RANGING_SENSOR_ConfigProfile(VL53L5A1_DEV_CENTER, &Profile);

  /* threshold parameters */
  ITConfig.Criteria = RS_IT_IN_WINDOW;
  ITConfig.LowThreshold = 100; /* mm */
  ITConfig.HighThreshold = 400; /* mm */

  VL53L5A1_RANGING_SENSOR_ConfigIT(VL53L5A1_DEV_CENTER, &ITConfig);

  status = VL53L5A1_RANGING_SENSOR_Start(VL53L5A1_DEV_CENTER, RS_MODE_ASYNC_CONTINUOUS);

  zones_per_line = ((Profile.RangingProfile == RS_PROFILE_8x8_AUTONOMOUS) ||
             (Profile.RangingProfile == RS_PROFILE_8x8_CONTINUOUS)) ? 8 : 4;

  if (status != BSP_ERROR_NONE)
  {
    printf("VL53L5A1_RANGING_SENSOR_Start failed\n");
    while(1);
  }

  neai_classification_init(knowledge);

  /* Reset ToF event variable to avoid starting log at boot */
  ToF_EventDetected = 0;

  while (1)
  {
    /* interrupt mode */
    if (ToF_EventDetected != 0) {
//    	uint32_t t0 = HAL_GetTick();
      for (uint8_t i = 0; i < SUCCESSIVE_MATRIX; i++) {
				do {
					status = VL53L5A1_RANGING_SENSOR_GetDistance(VL53L5A1_DEV_CENTER, &Result);
				} while(status != BSP_ERROR_NONE);
				for (uint8_t j = 0; j < Result.NumberOfZones; j += zones_per_line) {
					for (int8_t k = (zones_per_line - 1); k >= 0; k--) {
						if (Result.ZoneResult[j+k].NumberOfTargets > 0) {
							neai_buffer[(i * MATRIX_RESOLUTION) + j + k] = (float)Result.ZoneResult[j+k].Distance[0];
						}
						else {
							/* If no target detected we set data to max threshold */
							neai_buffer[(i * MATRIX_RESOLUTION) + j + k] = (float)ITConfig.HighThreshold;
						}
					}
				}
      }
//      uint32_t t1 = HAL_GetTick();
//      uint32_t time_ms = t1 - t0;
//      printf("Timer: %lu ms.\n", time_ms);
      /* Clean buffer : remove too high / low values which are not necessary
       * so we only keep the usefull data and convert them in binary data
       */
			for (uint16_t i = 0; i < SUCCESSIVE_MATRIX * MATRIX_RESOLUTION; i++) {
				if (neai_buffer[i] > ITConfig.HighThreshold || neai_buffer[i] < ITConfig.LowThreshold) {
					neai_buffer[i] = 0.0;
				}
				else {
					neai_buffer[i] = 1.0;
				}
			}
			if (NEAI_MODE) {
				/* Call NEAI classifier function in order to identify the phenomenom */
				neai_classification(neai_buffer, output_class_buffer, &id_class);
				printf("Class detected: %s.\n", id2class[id_class]);
			}
			else {
				/* Print the whole buffer to the serial in order to log data */
				for (uint16_t i = 0; i < SUCCESSIVE_MATRIX * MATRIX_RESOLUTION; i++) {
					printf("%.1f ", neai_buffer[i]);
				}
				printf("\n");
			}
      /* Reset ToF event variable when log has been done successfully */
      ToF_EventDetected = 0;
    }
  }
}

static void print_result(RANGING_SENSOR_Result_t *Result)
{
  int8_t i, j, k, l;
  uint8_t zones_per_line;

  zones_per_line = ((Profile.RangingProfile == RS_PROFILE_8x8_AUTONOMOUS) ||
         (Profile.RangingProfile == RS_PROFILE_8x8_CONTINUOUS)) ? 8 : 4;

  printf("%c[2H", 27); /* clear screen */
  printf("53L5A1 Threshold Detection demo application\n");
  printf("-------------------------------------------\n\n");

  printf("Cell Format :\n\n");
  for (l = 0; l < RANGING_SENSOR_NB_TARGET_PER_ZONE; l++)
  {
    printf(" \033[38;5;10m%20s\033[0m : %20s\n", "Distance [mm]", "Status");
    if ((Profile.EnableAmbient != 0) || (Profile.EnableSignal != 0))
    {
      printf(" %20s : %20s\n", "Signal [kcps/spad]", "Ambient [kcps/spad]");
    }
  }

  printf("\n\n");

  for (j = 0; j < Result->NumberOfZones; j += zones_per_line)
  {
    for (i = 0; i < zones_per_line; i++) /* number of zones per line */
      printf(" -----------------");
    printf("\n");

    for (i = 0; i < zones_per_line; i++)
      printf("|                 ");
    printf("|\n");

    for (l = 0; l < RANGING_SENSOR_NB_TARGET_PER_ZONE; l++)
    {
      /* Print distance and status */
      for (k = (zones_per_line - 1); k >= 0; k--)
      {
        if (Result->ZoneResult[j+k].NumberOfTargets > 0)
          printf("| \033[38;5;10m%5ld\033[0m  :  %5ld ", 
              (long)Result->ZoneResult[j+k].Distance[l],
              (long)Result->ZoneResult[j+k].Status[l]);
        else
          printf("| %5s  :  %5s ", "X", "X");
      }
      printf("|\n");

      if ((Profile.EnableAmbient != 0) || (Profile.EnableSignal != 0))
      {
        /* Print Signal and Ambient */
        for (k = (zones_per_line - 1); k >= 0; k--)
        {
          if (Result->ZoneResult[j+k].NumberOfTargets > 0)
          {
            if (Profile.EnableSignal != 0)
              printf("| %5ld  :  ", (long)Result->ZoneResult[j+k].Signal[l]);
            else
              printf("| %5s  :  ", "X");

            if (Profile.EnableAmbient != 0)
              printf("%5ld ", (long)Result->ZoneResult[j+k].Ambient[l]);
            else
              printf("%5s ", "X");
          }
          else
            printf("| %5s  :  %5s ", "X", "X");
        }
        printf("|\n");
      }
    }
  }

  for (i = 0; i < zones_per_line; i++)
    printf(" -----------------");
  printf("\n");
}

#ifdef __cplusplus
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
